package models

import (
	"github.com/beego/beego/v2/client/orm"
	"time"
)

// ComTradeCountry 联合国商品贸易数据库的国家编码表
type ComTradeCountry struct {
	Id                   int       `orm:"column(id);pk"`
	ThirdId              int       `description:"第三方的国家id"`
	Name                 string    `description:"国家名称（英文）"`
	NameCn               string    `description:"国家名称（中文）"`
	TogetherCountry      int       `description:"合并其他国家查询的个数，如果大于0，代表可以合并其他国家查询"`
	MonthTogetherCountry int       `description:"月度合并其他国家查询的个数，如果大于0，代表可以合并其他国家查询"`
	ModifyTime           time.Time `description:"修改时间"`
	CreateTime           time.Time `description:"创建时间"`
}

// Update 更新基础信息
func (item *ComTradeCountry) Update(cols []string) (err error) {
	o := orm.NewOrmUsingDB("data")
	_, err = o.Update(item, cols...)
	return
}

// AddComTradeCountry 添加联合国商品贸易数据库的国家编码
func AddComTradeCountry(item *ComTradeCountry) (lastId int64, err error) {
	o := orm.NewOrmUsingDB("data")
	lastId, err = o.Insert(item)
	if err != nil {
		return
	}
	item.Id = int(lastId)
	return
}

// GetComTradeCountryAll 获取所有的国家
func GetComTradeCountryAll() (list []*ComTradeCountry, err error) {
	o := orm.NewOrmUsingDB("data")
	sql := `SELECT * FROM com_trade_country where id !=1`
	_, err = o.Raw(sql).QueryRows(&list)
	return
}
